/*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 */

/* $Id: mconsole_c.c 187 2005-10-14 10:54:15Z gerd $ */

/* Linux: make all system prototypes available */
#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <stdint.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/uio.h>

#include "caml/mlvalues.h"
#include "caml/alloc.h"
#include "caml/fail.h"
#include "caml/memory.h"

#define MCONSOLE_MAGIC (0xcafebabe)
#define MCONSOLE_MAX_DATA (512)
#define MCONSOLE_VERSION (2)

struct mconsole_request {
        uint32_t magic;
        uint32_t version;
        uint32_t len;
        char data[MCONSOLE_MAX_DATA];
};

struct mconsole_reply {
        uint32_t err;
        uint32_t more;
        uint32_t len;
        char data[MCONSOLE_MAX_DATA];
};


value mconsole_mk_request (value cmdstring_val) {
    struct mconsole_request req;
    int l;
    CAMLparam1(cmdstring_val);
    CAMLlocal1(r);

    req.magic = MCONSOLE_MAGIC;
    req.version = MCONSOLE_VERSION;
    
    l = string_length(cmdstring_val);
    if (l >= MCONSOLE_MAX_DATA-1) {
	failwith("mconsole_mk_request: command too long");
    };

    req.len = l;
    strncpy(req.data, String_val(cmdstring_val), l+1);
    /* req.len+1: Includes the null byte after the string */

    r = alloc_string(sizeof(req));
    memcpy(String_val(r), (const char *) &req, sizeof(req));

    CAMLreturn(r);
}


value mconsole_dest_reply(value reply_val) {
    struct mconsole_reply reply;
    CAMLparam1(reply_val);
    CAMLlocal1(r);

    if (string_length(reply_val) < 13) {
	failwith("mconsole_dest_reply: string too short");
    };

    if (string_length(reply_val) > sizeof(reply)) {
	failwith("mconsole_dest_reply: string too long");
    };

    memcpy((char *) &reply, String_val(reply_val), string_length(reply_val));
    reply.data[string_length(reply_val) - 1] = 0;
    reply.data[MCONSOLE_MAX_DATA-1] = 0;

    r = alloc_tuple(3);
    Store_field(r, 0, reply.err ? Val_true : Val_false);
    Store_field(r, 1, reply.more ? Val_true : Val_false);
    Store_field(r, 2, copy_string(reply.data));

    CAMLreturn(r);
}
